    /*

    Copyright (C) 1999 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#include "md5auth.h"
#include "micosec.h"

Interceptor::Status MICOSec_SecurityChecker::initialize_request(
			Interceptor::LWServerRequest_ptr req,
            CORBA::Environment_ptr env)
{
   	CORBA::Object_var obj = req->target ();
    CORBA::Principal_var p = the_boa->get_principal (obj, env);

   	CORBA::Any_var a = p->get_property ("peer-info");
    CORBA::OctetSeq o;

  	if(a >>= o)
	{
		const char *cookie = md5_auth_cookie();

		if(o.length() && o[o.length()-1] == 0)
			if(strcmp(cookie,(char *)&o[0]) == 0)		// cookie matches?
				return Interceptor::INVOKE_CONTINUE;
	}

   	CORBA::Any_var pa = p->get_property ("peer-address");
	char *address;

	cerr << "MICOSec: Rejected unauthorized request ";
  	if(pa >>= address)
	{
		cerr << "from " << address;
		//CORBA::string_free(address);
	}
	cerr << "!" << endl;

	// permission denied ...
	env->exception (new CORBA::NO_PERMISSION());
	return Interceptor::INVOKE_ABORT;
}

void MICOSec_SecurityChecker::setBOA(CORBA::BOA_ptr boa)
{
	the_boa = boa;
}

MICOSec::MICOSec(CORBA::ORB_ptr orb, CORBA::BOA_ptr boa)
{
	const char *ma = getenv("MICOAUTHORITY");
	string temp;

	if(!ma) {
		const char *home = getenv("HOME");
		if(!home) {
			cerr << "Fatal error: Since HOME isn't set,"
					" .MICOauthority can't be found" << endl;
			exit(1);
		}
		temp = home;
		temp += "/.MICOauthority";

		ma = temp.c_str();
	}

	md5_auth_init(ma);

	const char *cookie = md5_auth_cookie();
	unsigned long l = strlen(cookie)+1;		// include a terminating 0 byte
	CORBA::OctetSeq os(l,l,(CORBA::Octet *)cookie,false);
	CORBA::Principal::set_peer_info(os);

	c.setBOA(boa);
	c.activate(0);
}
